<?php

namespace Drupal\layout_paragraphs_limit\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\layout_paragraphs\Event\LayoutParagraphsAllowedTypesEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Class LayoutParagraphsLimitAllowedTypesSubscriber.
 *
 * @package Drupal\layout_paragraphs_limit
 */
class LayoutParagraphsLimitAllowedTypesSubscriber implements EventSubscriberInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * LayoutParagraphsLimitAllowedTypesSubscriber constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   */
  public function __construct(ConfigFactoryInterface $configFactory) {
    $this->configFactory = $configFactory;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      LayoutParagraphsAllowedTypesEvent::EVENT_NAME => 'typeRestrictions',
    ];
  }

  /**
   * Implements typeRestrictions().
   *
   * @param \Drupal\layout_paragraphs\Event\LayoutParagraphsAllowedTypesEvent $event
   *   The event object.
   */
  public function typeRestrictions(LayoutParagraphsAllowedTypesEvent $event) {
    $parent_uuid = $event->getParentUuid();
    $parent = $event->getLayout()->getComponentByUuid($parent_uuid);
    if (!$parent) {
      return;
    }
    $parent_settings = $parent->getSettings();
    $parent_layout = $parent_settings['layout'];

    // Get the disallowed paragraph types for the current layout.
    $disallowed_types = $this->configFactory->get('layout_paragraphs_limit.settings')->get('disallowed_types');

    // Remove the disallowed paragraph types from the allowed types.
    $allowed_types = array_diff_key($event->getTypes(), $disallowed_types[$parent_layout]);

    // Set the updated allowed types.
    $event->setTypes($allowed_types);
  }

}
