<?php
namespace Drupal\layout_paragraphs_limit\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\paragraphs\Entity\ParagraphsType;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Layout\LayoutPluginManagerInterface;

/**
 * Class LayoutParagraphsLimitSettingsForm.
 *
 * @package Drupal\layout_paragraphs_limit\Form
 */
class LayoutParagraphsLimitSettingsForm extends ConfigFormBase {

  /**
   * The Layouts Manager.
   *
   * @var \Drupal\Core\Layout\LayoutPluginManagerInterface
   */
  protected $layoutPluginManager;

  /**
   * LayoutParagraphsLimitSettingsForm constructor.
   *
   * @param \Drupal\Core\Layout\LayoutPluginManagerInterface $layout_plugin_manager
   * Core layout plugin manager service.
   */
  public function __construct(LayoutPluginManagerInterface $layout_plugin_manager) {
    $this->layoutPluginManager = $layout_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.core.layout')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'layout_paragraphs_limit_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['layout_paragraphs_limit.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Get the available layout types.
    $layout_types = $this->layoutPluginManager->getLayoutOptions();

    // Get the available paragraph types.
    $paragraph_types = ParagraphsType::loadMultiple();

    // Get the disallowed types for each layout from the module's config.
    $config = $this->config('layout_paragraphs_limit.settings');
    $disallowed_types = $config->get('disallowed_types') ?: [];

    // Build the form.
    $form = parent::buildForm($form, $form_state);

    foreach ($layout_types as $group) {
      foreach ($group as $name => $label) {
        $form['disallowed_types'][$name] = [
          '#type' => 'select',
          '#multiple' => TRUE,
          '#title' => $this->t('Disallowed paragraph types for %layout (%layout_mn)', [
            '%layout' => $label,
            '%layout_mn' => $name,
          ]),
          '#options' => array_map(function ($type) {
            return $type->label();
          }, $paragraph_types),
          '#default_value' => isset($disallowed_types[$name]) ? $disallowed_types[$name] : [],
          '#parents' => ['disallowed_types', $name],
        ];
      }
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    // Save the disallowed types for each layout to the module's config.
    $config = $this->configFactory->getEditable('layout_paragraphs_limit.settings');
    $config->set('disallowed_types', $values['disallowed_types']);
    $config->save();

    parent::submitForm($form, $form_state);
  }

}
